package com.allpay.charge.aop;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @Description : 监控接口日志
 * @Author : Simon.shi
 * @Date : 2018/11/29 14:31
 * @Version : V0.0.1
 */
@Configuration
@Slf4j
@Aspect
public class WebLogAspect {

	@Pointcut(value = "@annotation(org.springframework.web.bind.annotation.RequestMapping)")
	public void webLog() {
	}

	@Around("webLog()")
	public Object doAround(ProceedingJoinPoint pjp) throws Throwable {
		RequestAttributes ra = RequestContextHolder.getRequestAttributes();
		ServletRequestAttributes sra = (ServletRequestAttributes) ra;
		HttpServletRequest request = sra.getRequest();
		String ip = null;
		//X-Forwarded-For：Squid 服务代理
		String ipAddresses = request.getHeader("X-Forwarded-For");

		if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
			//Proxy-Client-IP：apache 服务代理
			ipAddresses = request.getHeader("Proxy-Client-IP");
		}

		if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
			//WL-Proxy-Client-IP：weblogic 服务代理
			ipAddresses = request.getHeader("WL-Proxy-Client-IP");
		}

		if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
			//HTTP_CLIENT_IP：有些代理服务器
			ipAddresses = request.getHeader("HTTP_CLIENT_IP");
		}

		if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
			//X-Real-IP：nginx服务代理
			ipAddresses = request.getHeader("X-Real-IP");
		}

		//有些网络通过多层代理，那么获取到的ip就会有多个，一般都是通过逗号（,）分割开来，并且第一个ip为客户端的真实IP
		if (ipAddresses != null && ipAddresses.length() != 0) {
			ip = ipAddresses.split(",")[0];
		}
		//还是不能获取到，最后再通过request.getRemoteAddr();获取
		if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
			ip = request.getRemoteAddr();
		}

		String url = request.getRequestURL().toString();
		String method = request.getMethod();
		String uri = request.getRequestURI();
		String queryString = "";
		if (method.equalsIgnoreCase("GET") || method.equalsIgnoreCase("DELETE")) {
			queryString = request.getQueryString();
		} else {
			String header = request.getHeader("Content-Type");
			Map<String, String[]> map = request.getParameterMap();
			//通过keySet遍历集合
			Set<String> keySet = map.keySet();
			HashMap<String, String> hashMap = new HashMap<>();
			for (String key : keySet) {
				//通过key值获取所有value值
				String[] values = map.get(key);
				for (String value : values) {
					hashMap.put(key, value);
				}
			}
			queryString = hashMap.toString();
           /* if(header.equalsIgnoreCase("application/x-www-form-urlencoded")){
                queryString = request.getParameterMap().toString();
            }*/
		}
		log.info("请求开始============》ip: {},url: {}, method: {}, uri: {}, params: {}", ip, url, method, uri, queryString);

		// result的值就是被拦截方法的返回值
		Object result = pjp.proceed();
		ObjectMapper mapper = new ObjectMapper();
		log.info("请求结束============》接口的返回值是 " + mapper.writeValueAsString(result));
		return result;
	}

}
