package com.allpay.charge.interceptor;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpException;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.allpay.charge.annoation.GetUserInfo;
import com.allpay.charge.annoation.PermissionRequire;
import com.allpay.charge.calc.define.ResultVO;
import com.allpay.charge.exception.BadRequestException;
import com.allpay.charge.exception.UnauthenticatedException;
import com.allpay.charge.tools.SignatureUtil;
import com.allpay.charge.tools.ThreadLocalUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

/**
 * @Description :
 * @Author : Simon.shi
 * @Date : 2019/8/19 09:37
 * @Version : V0.0.1
 */
@Slf4j
@Component
public class PermissionsInterceptor extends HandlerInterceptorAdapter {

    @Value("${commonMessage.verifyFunctionUrl}")
    private String verifyFunctionUrl;
    @Value("${commonMessage.userInfo}")
    private String userInfoUrl;

    private boolean returnResult(HttpServletResponse response, String message) {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        try (PrintWriter out = response.getWriter()) {
            ResultVO result = new ResultVO();
            result.setStatus(ResultVO.FAIL);
            result.setMessage(message);
            String jsonString = JSON.toJSONString(result);
            out.append(jsonString);
            return false;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }


    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (handler instanceof HandlerMethod) {
            //签名验证
            boolean vertify = SignatureUtil.vertify(request, "3d98df38bd49ab8ac7e64f21197");
            if(!vertify){
                return false;
            }
            String authorization = request.getHeader("Authorization");
            PermissionRequire annotation = ((HandlerMethod) handler).getMethodAnnotation(PermissionRequire.class);
            GetUserInfo userInfoAnnotation = ((HandlerMethod) handler).getMethodAnnotation(GetUserInfo.class);
            if (ObjectUtil.isEmpty(userInfoAnnotation)) {
                String userInfo = HttpUtil.createGet(userInfoUrl).header("Authorization", authorization).execute().body();
                JSONObject jsonObject = JSON.parseObject(userInfo);
                if (ObjectUtil.isNotNull(jsonObject) && "success".equals(jsonObject.getString("status"))) {
                    String operator = jsonObject.getJSONObject("data").getString("operatorCode");
                    ThreadLocalUtil.operatorId.set(operator);
                } else {
                   // return returnResult(response, "获取用户信息失败");
                     throw new BadRequestException("获取用户信息失败");
                }
            }
            if (annotation != null) {
                String res = null;
                log.info("权限验证接口:{}", annotation.value());
                try {
                    res = HttpUtil.createGet(verifyFunctionUrl + "?function_id=" + annotation.value()).header("Authorization", authorization).execute().body();
                } catch (HttpException e) {
                    log.warn("请求权限异常:{}", e.getMessage());
                   // return returnResult(response, "权限接口异常");
                    throw new UnauthenticatedException("权限接口异常");
                }
                JSONObject jsonObject = JSON.parseObject(res);
                if (ObjectUtil.isNotNull(jsonObject) && "success".equals(jsonObject.getString("status"))) {
                    return true;
                } else {
                    //return returnResult(response, "没有权限");
                    throw new UnauthenticatedException("没有权限");
                }
            } else {
                return true;
            }
        }
        return true;
    }


}

