package com.allpay.charge.tools;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.parser.Feature;
import com.allpay.charge.calc.define.ResultVO;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author : lei.chen
 * @Company : AllPay
 * @CreateDate : 2019/5/16 9:32
 */
public class SignatureUtil {

	/**
	 * 日志层
	 */
	private static final Logger logger = LoggerFactory.getLogger(SignatureUtil.class);

	public static String getSignature(HashMap map, String SecretKey) {

		List<Map.Entry<String, String>> infoIds = new ArrayList<Map.Entry<String, String>>(map.entrySet());
		// 对所有传入参数按照字段名的 ASCII 码从小到大排序（字典序）
		// sort all incoming parameters by ASCII code of field name from small to large (dictionary order)
		Collections.sort(infoIds, new Comparator<Map.Entry<String, String>>() {
			@Override
			public int compare(Map.Entry<String, String> o1, Map.Entry<String, String> o2) {
				return (o1.getKey()).toString().compareTo(o2.getKey());
			}
		});
		// 构造签名键值对的格式
		//Construct a format for signing key-value pairs
		StringBuilder String1 = new StringBuilder();
		for (Map.Entry<String, String> item : infoIds) {
			if (item.getKey() != null && !"".equals(item.getKey())) {
				String val = item.getValue();
				String key = item.getKey();
				if (val != null) {
					String1.append(key + "=" + val + "&");
				}
			}

		}

		String1.deleteCharAt(String1.length() - 1);
		//添加双方都同意的签名密钥
		//the signed key agreed by both
		StringBuilder stringSignTemp = String1.append(SecretKey);
		// Add the signature key agreed by both parties
		String result = stringSignTemp.toString();
		//SHA256 encryption to get the signature
		String signature = SHA256Util.getSHA256(result);
		return signature;
	}

	public static boolean vertify(HttpServletRequest request, String SecretKey) {

		//将需要验签的参数转化成map集合形式开始验签
		//String json = JSON.toJSONString(object);
		//漏洞检查完之后再改回来
		//Map map=JSON.parseObject(json,new TypeReference<TreeMap<String, String>>(){} , Feature.OrderedField);
		Map map = converRequest(request);
		logger.info("request param:" + map);
		Object sign = map.get("signature");
		if (sign == null) {
			returnResult("signature is not exist");
			return false;
		}
		map.remove("signature");
		List<Map.Entry<String, String>> infoIds = new ArrayList<Map.Entry<String, String>>(map.entrySet());
		// 对所有传入参数按照字段名的 ASCII 码从小到大排序（字典序）
		// sort all incoming parameters by ASCII code of field name from small to large (dictionary order)
		Collections.sort(infoIds, new Comparator<Map.Entry<String, String>>() {
			@Override
			public int compare(Map.Entry<String, String> o1, Map.Entry<String, String> o2) {
				return (o1.getKey()).toString().compareTo(o2.getKey());
			}
		});
		// 构造签名键值对的格式
		//Construct a format for signing key-value pairs
		StringBuilder String1 = new StringBuilder();
		for (Map.Entry<String, String> item : infoIds) {
			if (item.getKey() != null && !"".equals(item.getKey()) && item.getValue() != null) {
				String key = item.getKey();
				Object val = item.getValue();
				if (val != null) {
					String1.append(key + "=" + val + "&");
				}
			}

		}
		if (StringUtils.isNotBlank(String1.toString())) {
			String1.deleteCharAt(String1.length() - 1);
		}
		//添加双方都同意的签名密钥
		//the signed key agreed by both
		StringBuilder stringSignTemp = String1.append(SecretKey);
		// Add the signature key agreed by both parties
		String result = stringSignTemp.toString();
		logger.info("signature parameter: " + result);
		//SHA256 encryption to get the signature
		String signature = SHA256Util.getSHA256(result);
		logger.info("right signature:" + signature);
		if (!sign.equals(signature)) {
			returnResult("signature is error");
			return false;
		}
			return true;
	}

	private static void returnResult(String message) {
		HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
		response.setCharacterEncoding("UTF-8");
		response.setContentType("application/json; charset=utf-8");
		try (PrintWriter out = response.getWriter()) {
			ResultVO result = new ResultVO();
			result.setStatus(ResultVO.FAIL);
			result.setMessage(message);
			String jsonString = JSON.toJSONString(result);
			out.append(jsonString);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static Map converRequest(HttpServletRequest request){
		Map properties = request.getParameterMap();
		Map returnMap = new HashMap();
		Iterator entries = properties.entrySet().iterator();
		Map.Entry entry;
		String name = "";
		String value = "";
		while (entries.hasNext()) {
			entry = (Map.Entry) entries.next();
			name = (String) entry.getKey();
			Object valueObj = entry.getValue();
			if(null == valueObj){
				value = "";
			}else if(valueObj instanceof String[]){
				String[] values = (String[])valueObj;
				for(int i=0;i<values.length;i++){
					value = values[i] + ",";
				}
				value = value.substring(0, value.length()-1);
			}else{
				value = valueObj.toString();
			}
			returnMap.put(name, value);
		}
		return returnMap;
	}
}
